////////////////////////////////////////////////////////////////////////////////
//
// @copyright COSEDA Technologies GmbH. All rights reserved.
//            (customizable in sca_modules.dtd)
//
////////////////////////////////////////////////////////////////////////////////
//
// @project      pll
//
// @library      libpll
//
// @file         charge_pump
//
// @brief
//      /** Charge pumo module */
//
// DESCRIPTION OF PORTS:           (please see header)
//
// this file was generated by 'sca_xml2cpp.xsl' StyleSheet
//
////////////////////////////////////////////////////////////////////////////////
//
// Version Id       : $Id$
// @author          : Am�rico Dias
// Last Modified By : $Author$
// Last Modified On : $Date$
//
////////////////////////////////////////////////////////////////////////////////

#ifndef PLL_LIBPLL_CP_H_
#include "cp.h"
#define COSIDE_INCLUDE_IMPLEMENTATION
#endif

// adds SystemC namespaces for user convenience
#include <systemc-ams.h>

namespace pll_namespace
{

//-------------------------------------------------------------------//
// states                                                            //
//-------------------------------------------------------------------//
struct cp::states
{

    states(const params& p)
    {
    }

};
//-------------------------------------------------------------------//

#ifdef COSIDE_INCLUDE_IMPLEMENTATION


//-------------------------------------------------------------------//
cp::states& cp::create_states(const params& pa) { return *(new states(pa)); }
//-------------------------------------------------------------------//


//////////////////////////////////////////////
// method called by constructor             //
//////////////////////////////////////////////
void cp::construct()
{

}

//////////////////////////////////////////////
// destructor                               //
//////////////////////////////////////////////
cp::~cp()
{

    delete &s;
}

//////////////////////////////////////////////
// method processing                        //
//////////////////////////////////////////////
void cp::processing()
{
	double charge = 0,
			discharge = 0,
			mismatch = 0,
			i = 0,
			mosfet_vtp=p.mosfet_vth,
			mosfet_vtn=p.vdd-p.mosfet_vth;

	// Discharge (NMOS)
	if(vcp>mosfet_vtn) {
		// Saturation
		discharge=-p.current_dn;
	} else {
		// Triode
		double vds = vcp;
		discharge=-((2.0/(p.vdd-p.mosfet_vth))*p.current_dn*vds-(1.0/pow(p.vdd-p.mosfet_vth,2.0))*p.current_dn*pow(vds,2.0));
	}

	// Charge (PMOS)
	if(vcp<mosfet_vtp) {
		// Saturation
		charge=p.current_up;
	} else {
		// Triode
		double vds = p.vdd-vcp;
		charge=(2.0/(p.vdd-p.mosfet_vth))*p.current_up*vds-(1.0/pow(p.vdd-p.mosfet_vth,2.0))*p.current_up*pow(vds,2.0);
	}

	mismatch=charge+discharge;


	/**
	 * Set the output current depending on the inputs state
	 * (up/down)
	 */
	if(up && !dn) {
		i = charge;
	}
	else if(!up && dn) {
		i = discharge;
	}
	else if(!up && !dn) {
		i = -p.current_leak;
	}
	else if(up && dn) {
		i = mismatch;
	}

	/**
	 * Limit the output current if the loop filter input voltage
	 * is out of limits (V_CTRL < 0 || V_CTRL > vdd)
	 */
	if((vcp>p.vdd && i>0) || (vcp<0 && i<0)) {
		i=0;
	}

	icp = i;
}

//////////////////////////////////////////////
// method called by set_attributes          //
//////////////////////////////////////////////
void cp::set_attributes_cpp()
{
	icp.set_timestep(p.ts,sc_core::SC_SEC);
	icp.set_delay(1);
}



#endif // #ifdef COSIDE_INCLUDE_IMPLEMENTATION

}  // end namespace pll_namespace

// clear temporary defines
#undef COSIDE_INCLUDE_IMPLEMENTATION
