/////////////////////////////////////////////////////////////////////////////////
///
/// @copyright COSEDA Technologies GmbH. All rights reserved.
///            (customizable in sca_modules.dtd)
///
/////////////////////////////////////////////////////////////////////////////////
//
// @project      pll
//
// @library      libpll
//
// @file         vco
//
// @brief
//      /** Voltage controlled oscillator implementation */
//
// DESCRIPTION OF PORTS:
// vctrl of type double
//       /** Control voltage */
// fout of type double
//       /** Output signal */
// freq of type double
//       /** Frequency of the output signal */
//
// DESCRIPTION OF PARAMETERS:
// tstep of type double
//       /** Time step */
// vdd of type double
//       /** Voltage Supply */
// vcm of type double
//       /** Common Mode Voltage */
// kvo of type double
//       /** Gain */
// fmin of type double
//       /** Minimum oscilation frequency for vctrl=0 */
//
// this file was generated by 'sca_xml2h.xsl' StyleSheet
//
////////////////////////////////////////////////////////////////////////////////
//
//  Version Id       : $Id$
//  Last Modified By : $Author$
//  Last Modified On : $Date$
//
////////////////////////////////////////////////////////////////////////////////

#ifndef PLL_LIBPLL_VCO_H_
#define PLL_LIBPLL_VCO_H_

/** ============================ DO NOT EDIT THIS FILE! ========================
 *  = This file was automatically generated from an COSIDE primitive.
 *  = (do changes only via the module editor!)
 *  ============================================================================
 */

// 29_BEGIN_USER_CODE
// you can add own code here.
// END_USER_CODE_29

#include <systemc-ams>

// 30_BEGIN_USER_CODE
// you can add own code here.
// END_USER_CODE_30


namespace pll_namespace
{

// 31_BEGIN_USER_CODE
// you can add own code here.
// END_USER_CODE_31

SCA_TDF_MODULE(vco)
{
    // 32_BEGIN_USER_CODE
    // you can add own code here.
    // END_USER_CODE_32

    /// sca tdf ports
    sca_tdf::sca_in<double  > vctrl; /** Control voltage */
    sca_tdf::sca_out<double  > fout; /** Output signal */
    sca_tdf::sca_out<double  > freq; /** Frequency of the output signal */

    /// parameters
    const struct params
    {
        // 34_BEGIN_USER_CODE
        // you can add own code here.
        // END_USER_CODE_34
        double tstep; /** Time step */
        double vdd; /** Voltage Supply */
        double vcm; /** Common Mode Voltage */
        double kvo; /** Gain */
        double fmin; /** Minimum oscilation frequency for vctrl=0 */

        params()
        {
            // 35_BEGIN_USER_CODE
            // you can add own code here.
            // END_USER_CODE_35
            vdd = 3.3;
            vcm = 1.65;
        }
    } p;


    /// methods
    /**
     * SystemC AMS processing callback see LRM
     */
    void processing() override;

    /// user methods

    private:
        /**
         * Circular Integrator Operator
         */
        double  idtmod(double frequency, double time);

    public:


    /**
     * SystemC AMS set_attributes callback see LRM
     */
    void set_attributes() override
    {
        this->set_attributes_cpp();
    }

    /// constructor
    vco(sc_core::sc_module_name, const params& pa = params() ) : vctrl("vctrl"),
                                                       fout("fout"),
                                                       freq("freq"),
                                                       p(pa),
                                                       s(this->create_states(pa))
    {


        this->construct();

    }

    /// destructor
    virtual ~vco();


    /// states
    struct states;
    states& s;
    states* operator -> () { return &s; }

    private:
        static states& create_states(const params& pa);
        void construct();
        void set_attributes_cpp();

        // 37_BEGIN_USER_CODE
        // you can add own code here.
        // END_USER_CODE_37

};


// 38_BEGIN_USER_CODE
// you can add own code here.
// END_USER_CODE_38

} // end namespace pll_namespace
using pll_namespace::vco;

#endif // PLL_LIBPLL_VCO_H_

#ifdef ACCESS_MODULE_STATE_DECLARATION
#ifndef PLL_LIBPLL_VCO_CPP_
#define PLL_LIBPLL_VCO_CPP_
#include "vco.cpp"
#endif
#endif
